/*
 * MakroAction.hpp
 *
 *  Created on: Dec 17, 2009
 *      Author: crueger
 */

#ifndef MAKROACTION_HPP_
#define MAKROACTION_HPP_

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <string>

#include "Actions/Action.hpp"

namespace MoleCuilder {
  class ActionRegistry;
  class ActionSequence;

/**
 * Action to allow producing bigger Actions from sequences of small actions.
 *
 * Destruction of the Actions and the sequence is handled by this class.
 */
class MakroAction : public Action
{
public:
  MakroAction(const ActionTrait &_trait,ActionSequence& _actions);
  virtual ~MakroAction();

  bool canUndo();
  bool shouldUndo();

  virtual void outputAsCLI(std::ostream &ost) const;
  virtual void outputAsPython(std::ostream &ost, const std::string &prefix) const;

  // must be called after all primitive actions are present
  virtual void prepare(ActionRegistry &AR);
  // must be called before alle primitive actions are removed
  virtual void unprepare(ActionRegistry &AR);

protected:
  /**
   * MakroAction requires an own dialog for global options such as number of loop
   * iterations, ... besides the dialog from each action in the sequence.
   *
   * However, fillDialog() calls ActionSequence::fillAllDialog(). Hence, the dialog
   * for specific global options must be requested in an distinct function.
   */
  virtual Dialog * fillOwnDialog(Dialog*);

  /** Pass-thru for removeAction to allow all derived MakroActions to manipulate action sequence.
   *
   * \param name name of action to remove
   * \return true - action removed, false - action not found
   * \sa ActionSequence::removeAction()
   */
  bool removeAction(const std::string &name);

  virtual ActionState::ptr performCall();
  virtual ActionState::ptr performUndo(ActionState::ptr);
  virtual ActionState::ptr performRedo(ActionState::ptr);

  void setLoop(unsigned int _loop);

private:
  /** fillDialog is used to spawn sequence queries, hence must not be modified by
   * derived MakroActions.
   */
  Dialog *fillDialog(Dialog *dialog);

  ActionSequence &actions;
};

}

#endif /* MAKROACTION_HPP_ */
